@@ -1,5 +1,5 @@
 #
-# Copyright (c), 2018-2020, SISSA (International School for Advanced Studies).
+# Copyright (c), 2018-2025, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -7,21 +7,22 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-# type: ignore
 """
 XPath 3.0 implementation - part 2 (symbols, operators and expressions)
 """
 from copy import copy
+from collections.abc import Iterator
+from typing import Any, cast, Union
 
-from ..namespaces import XPATH_FUNCTIONS_NAMESPACE, XSD_NAMESPACE
-from ..xpath_nodes import AttributeNode, ElementNode
-from ..xpath_tokens import XPathToken, ValueToken, XPathFunction, \
+import elementpath.aliases as ta
+import elementpath.namespaces as ns
+from elementpath.datatypes import QName
+from elementpath.xpath_tokens import XPathToken, ValueToken, XPathFunction, \
     XPathMap, XPathArray
-from ..xpath_context import XPathSchemaContext
-from ..datatypes import QName
 
 from .xpath30_parser import XPath30Parser
 
+__all__ = ['XPath30Parser']
 
 register = XPath30Parser.register
 infix = XPath30Parser.infix
@@ -37,12 +38,12 @@ register('?', bases=(ValueToken,))
 
 
 @method('?')
-def nud_placeholder_symbol(self):
+def nud__placeholder_symbol(self: ValueToken) -> ValueToken:
     return self
 
 
 @method('?')
-def evaluate_placeholder_symbol(self, context=None):
+def evaluate__placeholder_symbol(self: ValueToken, context: ta.ContextType = None) -> ValueToken:
     return self
 
 
@@ -60,7 +61,7 @@ XPath30Parser.unregister('(')
 
 
 @method(register('(', lbp=80, rpb=80, label='expression'))
-def nud_parenthesized_expression(self):
+def nud__parenthesized_expression(self: XPathToken) -> XPathToken:
     if self.parser.next_token.symbol != ')':
         self[:] = self.parser.expression(),
     self.parser.advance(')')
@@ -68,7 +69,7 @@ def nud_parenthesized_expression(self):
 
 
 @method('(')
-def led_parenthesized_expression(self, left):
+def led__parenthesized_expression(self: XPathToken, left: XPathToken) -> XPathToken:
     if left.symbol in ('(name)', 'Q{'):
         if left.value in self.parser.RESERVED_FUNCTION_NAMES:
             msg = f"{left.value!r} is not allowed as function name"
@@ -77,7 +78,7 @@ def led_parenthesized_expression(self, left):
             raise left.error('XPST0017', 'unknown function {!r}'.format(left.value))
 
     elif left.symbol == ':' and left[1].symbol == '(name)':
-        if left[1].namespace == XSD_NAMESPACE:
+        if left[1].namespace == ns.XSD_NAMESPACE:
             msg = 'unknown constructor function {!r}'.format(left[1].value)
             raise left[1].error('XPST0017', msg)
         raise left.error('XPST0017', 'unknown function {!r}'.format(left.value))
@@ -91,16 +92,19 @@ def led_parenthesized_expression(self, left):
 
 
 @method('(')
-def evaluate_parenthesized_expression(self, context=None):
+def evaluate__parenthesized_expression(self: XPathToken, context: ta.ContextType = None) \
+        -> ta.ValueType | XPathToken:
     if not self:
         return []
 
     value = self[0].evaluate(context)
+
     if isinstance(value, list) and len(value) == 1:
         value = value[0]
 
     if len(self) > 1:
         if isinstance(value, XPathFunction):
+            func: XPathFunction
             func = value
             tokens = self[1].get_argument_tokens()
 
@@ -111,13 +115,14 @@ def evaluate_parenthesized_expression(self, context=None):
                 func.to_partial_function()
                 return func
 
-            arguments = [tk.evaluate(context) for tk in tokens]
+            arguments: list[ta.ValueType] = [tk.evaluate(context) for tk in tokens]
+
             if func.label == 'partial function' and func[0].symbol == '?' and len(func[0]):
                 if context is None:
                     raise self.missing_context()
-                return func(context, context.item, *arguments)
+                return func(context.item, *arguments, context=context)
 
-            return func(context, *arguments)
+            return func(*arguments, context=context)
 
         elif self[0].symbol == '(':
             if not isinstance(value, list):
@@ -134,34 +139,32 @@ def evaluate_parenthesized_expression(self, context=None):
             not isinstance(value, XPathFunction) or self[0].span[0] > self.span[0]:
         return value
     else:
-        return value(context)
+        return value(context=context)
 
 
 @method(infix('||', bp=32))
-def evaluate_union_operator(self, context=None):
+def evaluate__union_operator(self: XPathToken, context: ta.ContextType = None) -> str:
 
     return self.string_value(self.get_argument(context)) + \
         self.string_value(self.get_argument(context, index=1))
 
 
 @method(infix('!', bp=72))
-def select_simple_map_operator(self, context=None):
+def select__simple_map_operator(self: XPathToken, context: ta.ContextType = None) \
+        -> Iterator[ta.ItemType]:
     if context is None:
         raise self.missing_context()
 
-    for context.item in context.inner_focus_select(self[0]):
-        for result in self[1].select(copy(context)):
+    for context.item in self[0].select_with_focus(context):
+        for result in self[1].select(context):
             yield result
-            if isinstance(context, XPathSchemaContext) and \
-                    isinstance(result, (AttributeNode, ElementNode)):
-                self[1].add_xsd_type(result)
 
 
 ###
 # 'let' expressions
 
-@method(register('let', lbp=20, rbp=20, label='let expression'))
-def nud_let_expression(self):
+@method('let', bp=20, label='expression')
+def nud__let_expression(self: XPathToken) -> XPathToken:
     del self[:]
     if self.parser.next_token.symbol != '$':
         return self.as_name()
@@ -183,12 +186,16 @@ def nud_let_expression(self):
 
 
 @method('let')
-def select_let_expression(self, context=None):
+def select__let_expression(self: XPathToken, context: ta.ContextType = None) \
+        -> Iterator[ta.ItemType]:
     if context is None:
         raise self.missing_context()
 
+    context = copy(context)
+    context.variables = context.variables.copy()
+
     for k in range(0, len(self) - 1, 2):
-        varname = self[k][0].value
+        varname = cast(str, self[k][0].value)
         value = self[k+1].evaluate(context)
         context.variables[varname] = value
 
@@ -196,7 +203,7 @@ def select_let_expression(self, context=None):
 
 
 @method('#', bp=90)
-def led_function_reference(self, left):
+def led__function_reference(self: XPathToken, left: XPathToken) -> XPathToken:
     if not left.label.endswith('function'):
         left.expected(':', '(name)', 'Q{')
 
@@ -206,31 +213,48 @@ def led_function_reference(self, left):
 
 
 @method('#')
-def evaluate_function_reference(self, context=None):
+def evaluate__function_reference(self: XPathToken, context: ta.ContextType = None) -> XPathFunction:
+    token_class: type[Union[XPathFunction, XPathToken]]
+    namespace: Any
+    name: Any
+
     arity = self[1].value
+    assert arity is None or isinstance(arity, int)
+
     if isinstance(self[0], XPathFunction):
         token_class = self[0].__class__
         namespace = self[0].namespace
+        name = self[0].name
+        if isinstance(name, QName):
+            qname = name
+        else:
+            qname = QName(None, f'anonymous {self[0].label}'.replace(' ', '-'))
     else:
         if self[0].symbol == ':':
-            qname = QName(self[0][1].namespace, self[0].value)
+            namespace = self[0][1].namespace
+            name = self[0].value
         elif self[0].symbol == 'Q{':
-            qname = QName(self[0][0].value, self[0][1].value)
-        elif self[0].value in self.parser.RESERVED_FUNCTION_NAMES:
+            namespace = self[0][0].value
+            name = self[0][1].value
+        elif self[0].value not in self.parser.RESERVED_FUNCTION_NAMES:
+            namespace = ns.XPATH_FUNCTIONS_NAMESPACE
+            name = self[0].value
+        else:
             msg = f"{self[0].value!r} is not allowed as function name"
             raise self.error('XPST0003', msg)
-        else:
-            qname = QName(XPATH_FUNCTIONS_NAMESPACE, self[0].value)
 
+        assert isinstance(name, str)
+        assert isinstance(namespace, str) or namespace is None
+        qname = QName(namespace, name)
         namespace = qname.namespace
         local_name = qname.local_name
 
         # Generic rule for XSD constructor functions
-        if namespace == XSD_NAMESPACE and arity != 1:
+        if namespace == ns.XSD_NAMESPACE and arity != 1:
             raise self.error('XPST0017', f"unknown function {qname.qname}#{arity}")
 
         # Special checks for multirole tokens
-        if namespace == XPATH_FUNCTIONS_NAMESPACE and \
+        if namespace == ns.XPATH_FUNCTIONS_NAMESPACE and \
                 local_name in ('QName', 'dateTime') and arity == 1:
             raise self.error('XPST0017', f"unknown function {qname.qname}#{arity}")
 
@@ -245,6 +269,7 @@ def evaluate_function_reference(self, context=None):
 
         if token_class.symbol == 'function' or not token_class.label.endswith('function'):
             raise self.error('XPST0003')
+        assert issubclass(token_class, XPathFunction)
 
     try:
         func = token_class(self.parser, nargs=arity)
