@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright (c), 2021, SISSA (International School for Advanced Studies).
+# Copyright (c), 2021-2025, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -22,8 +22,8 @@ except (ImportError, AttributeError):
     xmlschema = None
 
 
-@unittest.skipIf(xmlschema is None, "xmlschema library required.")
-class XMLSchemaProxyTest(unittest.TestCase):
+@unittest.skipIf(xmlschema is None, "xmlschema library required")
+class XMLSchemaContextTest(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
@@ -53,7 +53,6 @@ class XMLSchemaProxyTest(unittest.TestCase):
 
         elem_a = self.schema1.elements['a']
         token = parser.parse('a')
-        self.assertIsNone(token.xsd_types)
 
         context = copy(schema_context)
         element_node = context.root[0]
@@ -61,13 +60,10 @@ class XMLSchemaProxyTest(unittest.TestCase):
         self.assertIs(element_node.xsd_type, elem_a.type)
 
         result = token.evaluate(context)
-        self.assertEqual(token.xsd_types, {"{http://xpath.test/ns}a": elem_a.type})
-        self.assertListEqual(result, [element_node])
+        self.assertEqual(result, [element_node])
 
         elem_b1 = elem_a.type.content[0]
         token = parser.parse('a/b1')
-        self.assertIsNone(token[0].xsd_types)
-        self.assertIsNone(token[1].xsd_types)
 
         context = copy(schema_context)
         element_node = context.root[0][0]
@@ -75,70 +71,48 @@ class XMLSchemaProxyTest(unittest.TestCase):
         self.assertIs(element_node.xsd_type, elem_b1.type)
 
         result = token.evaluate(context)
-        self.assertEqual(token[0].xsd_types, {"{http://xpath.test/ns}a": elem_a.type})
-        self.assertEqual(token[1].xsd_types, {"b1": elem_b1.type})
-        self.assertListEqual(result, [element_node])
+        self.assertEqual(result, [element_node])
 
     def test_colon_token(self):
         parser = XPath2Parser(namespaces={'tst': "http://xpath.test/ns"})
         context = XPathSchemaContext(self.schema1)
 
-        elem_a = self.schema1.elements['a']
         token = parser.parse('tst:a')
         self.assertEqual(token.symbol, ':')
-        self.assertIsNone(token.xsd_types)
 
         result = token.evaluate(copy(context))
-        self.assertEqual(token.xsd_types, {"{http://xpath.test/ns}a": elem_a.type})
-        self.assertListEqual(result, [context.root[0]])
+        self.assertEqual(result, [context.root[0]])
 
-        elem_b1 = elem_a.type.content[0]
         token = parser.parse('tst:a/b1')
         self.assertEqual(token.symbol, '/')
         self.assertEqual(token[0].symbol, ':')
-        self.assertIsNone(token[0].xsd_types)
-        self.assertIsNone(token[1].xsd_types)
 
         result = token.evaluate(copy(context))
-        self.assertListEqual(result, [context.root[0][0]])
-        self.assertEqual(token[0].xsd_types, {"{http://xpath.test/ns}a": elem_a.type})
-        self.assertEqual(token[1].xsd_types, {"b1": elem_b1.type})
+        self.assertEqual(result, [context.root[0][0]])
 
         token = parser.parse('tst:a/tst:b1')
         result = token.evaluate(copy(context))
-        self.assertListEqual(result, [])
-        self.assertEqual(token[0].xsd_types, {"{http://xpath.test/ns}a": elem_a.type})
-        self.assertIsNone(token[1].xsd_types)
+        self.assertEqual(result, [])
 
-        elem_b3 = elem_a.type.content[2]
         token = parser.parse('tst:a/tst:b3')
         self.assertEqual(token.symbol, '/')
         self.assertEqual(token[0].symbol, ':')
-        self.assertIsNone(token[0].xsd_types)
-        self.assertIsNone(token[1].xsd_types)
 
         result = token.evaluate(copy(context))
-        self.assertListEqual(result, [context.root[0][2]])
-        self.assertEqual(token[0].xsd_types, {"{http://xpath.test/ns}a": elem_a.type})
-        self.assertEqual(token[1].xsd_types, {"{http://xpath.test/ns}b3": elem_b3.type})
+        self.assertEqual(result, [context.root[0][2]])
 
     def test_extended_name_token(self):
         parser = XPath2Parser(strict=False)
         context = XPathSchemaContext(self.schema1)
 
-        elem_a = self.schema1.elements['a']
         token = parser.parse('{http://xpath.test/ns}a')
         self.assertEqual(token.symbol, '{')
-        self.assertIsNone(token.xsd_types)
         self.assertEqual(token[0].symbol, '(string)')
         self.assertEqual(token[1].symbol, '(name)')
         self.assertEqual(token[1].value, 'a')
 
         result = token.evaluate(context)
-        self.assertListEqual(result, [context.root[0]])
-        self.assertEqual(token.xsd_types, {"{http://xpath.test/ns}a": elem_a.type})
-        self.assertIsNone(token[0].xsd_types)
-        self.assertIsNone(token[1].xsd_types)
+        self.assertEqual(result, [context.root[0]])
 
     def test_wildcard_token(self):
         parser = XPath2Parser(default_namespace="http://xpath.test/ns")
@@ -148,12 +122,9 @@ class XMLSchemaProxyTest(unittest.TestCase):
         elem_b3 = self.schema1.elements['b3']
         token = parser.parse('*')
         self.assertEqual(token.symbol, '*')
-        self.assertIsNone(token.xsd_types)
 
         result = token.evaluate(context)
-        self.assertListEqual([e.value for e in result], [elem_a, elem_b3])
-        self.assertEqual(token.xsd_types, {"{http://xpath.test/ns}a": elem_a.type,
-                                           "{http://xpath.test/ns}b3": elem_b3.type})
+        self.assertEqual([e.value for e in result], [elem_a, elem_b3])
 
         token = parser.parse('a/*')
         self.assertEqual(token.symbol, '/')
@@ -161,47 +132,34 @@ class XMLSchemaProxyTest(unittest.TestCase):
         self.assertEqual(token[1].symbol, '*')
 
         result = token.evaluate(context)
-        self.assertListEqual([e.value for e in result], elem_a.type.content[:])
-        self.assertIsNone(token.xsd_types)
-        self.assertEqual(token[0].xsd_types, {"{http://xpath.test/ns}a": elem_a.type})
-        self.assertEqual(token[1].xsd_types, {'b1': elem_a.type.content[0].type,
-                                              'b2': elem_a.type.content[1].type,
-                                              '{http://xpath.test/ns}b3': elem_b3.type})
+        self.assertEqual([e.value for e in result], elem_a.type.content[:])
 
     def test_dot_shortcut_token(self):
         parser = XPath2Parser(default_namespace="http://xpath.test/ns")
         context = XPathSchemaContext(self.schema1)
 
-        elem_a = self.schema1.elements['a']
-        elem_b3 = self.schema1.elements['b3']
-
         token = parser.parse('.')
-        self.assertIsNone(token.xsd_types)
         result = token.evaluate(context)
-        self.assertListEqual(result, [context.root])
-        self.assertEqual(token.xsd_types, {"{http://xpath.test/ns}a": elem_a.type,
-                                           "{http://xpath.test/ns}b3": elem_b3.type})
+        self.assertEqual(result, context.root)
 
         context = XPathSchemaContext(self.schema1, item=self.schema1)
         token = parser.parse('.')
-        self.assertIsNone(token.xsd_types)
         result = token.evaluate(context)
-        self.assertListEqual(result, [context.root])
-        self.assertEqual(token.xsd_types, {"{http://xpath.test/ns}a": elem_a.type,
-                                           "{http://xpath.test/ns}b3": elem_b3.type})
+        self.assertEqual(result, context.root)
 
         context = XPathSchemaContext(self.schema1, item=self.schema2)
         schema2_node = context.item
         token = parser.parse('.')
-        self.assertIsNone(token.xsd_types)
         result = token.evaluate(context)
-        self.assertListEqual(result, [schema2_node])
-        self.assertIsNone(token.xsd_types)
+        self.assertEqual(result, schema2_node)
 
     def test_schema_variables(self):
         variable_types = {'a': 'item()', 'b': 'xs:integer?', 'c': 'xs:string'}
-        parser = XPath2Parser(default_namespace="http://xpath.test/ns",
-                              variable_types=variable_types)
+        parser = XPath2Parser(
+            default_namespace="http://xpath.test/ns",
+            variable_types=variable_types,
+            schema=self.schema1.xpath_proxy,
+        )
         context = XPathSchemaContext(self.schema1)
 
         token = parser.parse('$a')
@@ -220,27 +178,38 @@ class XMLSchemaProxyTest(unittest.TestCase):
         self.assertEqual(result, '  alpha\t')
 
         token = parser.parse('$z')
-        with self.assertRaises(NameError):
-            token.evaluate(context)
+        self.assertEqual(token.evaluate(context), [])
 
     def test_not_applicable_functions(self):
         parser = XPath2Parser(default_namespace="http://xpath.test/ns")
         context = XPathSchemaContext(self.schema1)
 
         token = parser.parse("fn:collection('filepath')")
-        self.assertIsNone(token.evaluate(context))
+        self.assertEqual(token.evaluate(context), [])
 
         token = parser.parse("fn:doc-available('tns1')")
-        self.assertIsNone(token.evaluate(context))
+        self.assertFalse(token.evaluate(context))
 
         token = parser.parse("fn:root(.)")
-        self.assertIsNone(token.evaluate(context))
+        self.assertEqual(token.evaluate(context), [])
 
         token = parser.parse("fn:id('ID21256')")
-        self.assertListEqual(token.evaluate(context), [])
+        self.assertEqual(token.evaluate(context), [])
 
         token = parser.parse("fn:idref('ID21256')")
-        self.assertListEqual(token.evaluate(context), [])
+        self.assertEqual(token.evaluate(context), [])
+
+    def test_if_statement(self):
+        parser = XPath2Parser(default_namespace="http://xpath.test/ns")
+        context = XPathSchemaContext(self.schema1)
+
+        token = parser.parse('if ($x > 1) then a/b1 else a/b2')
+        result = token.evaluate(context)
+        self.assertEqual(result, [context.root[0][1]])
+
+        token = parser.parse('if ($x > xs:date("2010-01-01")) then a/b1 else a/b2')
+        result = token.evaluate(context)
+        self.assertEqual(result, [context.root[0][1]])
 
 
 if __name__ == '__main__':
