@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright (c), 2018-2020, SISSA (International School for Advanced Studies).
+# Copyright (c), 2018-2025, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -494,6 +494,8 @@ class XMLSchemaProxyTest(xpath_test_class.XPathTestCase):
         for node in context.root.iter_lazy():
             if isinstance(node, ElementNode):
                 self.assertIsNotNone(node.xsd_type)
+                self.assertIsNotNone(node.xsd_element)
+                self.assertEqual(node.name, node.xsd_element.name)
 
         self.assertEqual(context.root.children[0].type_name,
                          '{http://www.w3.org/2001/XMLSchema}string')
@@ -519,6 +521,8 @@ class XMLSchemaProxyTest(xpath_test_class.XPathTestCase):
         context.root.apply_schema(schema_proxy)
         self.assertEqual(context.root.children[0].type_name,
                          '{http://www.w3.org/2001/XMLSchema}integer')
+        self.assertEqual(context.root.children[0].xsd_element.name,
+                         '{http://xpath.test/ns}a')
         self.assertIsNone(context.root.children[0].attributes[0].xsd_type)  # not found
 
         xml_data = '<values xmlns="http://xpath.test/ns"><ax min="19">80</ax></values>'
@@ -529,6 +533,8 @@ class XMLSchemaProxyTest(xpath_test_class.XPathTestCase):
         context.root.apply_schema(schema_proxy)
         self.assertEqual(context.root.children[0].type_name,
                          '{http://xpath.test/ns}rangeType')
+        self.assertEqual(context.root.children[0].xsd_element.name,
+                         '{http://xpath.test/ns}ax')
         self.assertEqual(context.root.children[0].attributes[0].type_name,
                          '{http://www.w3.org/2001/XMLSchema}integer')
 
@@ -544,8 +550,10 @@ class XMLSchemaProxyTest(xpath_test_class.XPathTestCase):
                          '{http://xpath.test/ns}paramsType')
         self.assertEqual(context.root.children[0].children[0].type_name,
                          '{http://www.w3.org/2001/XMLSchema}float')
+        self.assertEqual(context.root.children[0].children[0].xsd_element.name, 'p0')
         self.assertEqual(context.root.children[0].children[1].type_name,
                          '{http://www.w3.org/2001/XMLSchema}untyped')
+        self.assertIsNone(context.root.children[0].children[1].xsd_element)
 
         xml_data = ('<p:values xmlns:p="http://xpath.test/ns">'
                     '<p:cx><p0>1.0</p0><p1>foo</p1></p:cx></p:values>')
@@ -560,6 +568,7 @@ class XMLSchemaProxyTest(xpath_test_class.XPathTestCase):
                          '{http://www.w3.org/2001/XMLSchema}float')
         self.assertEqual(context.root.children[0].children[1].type_name,
                          '{http://www.w3.org/2001/XMLSchema}string')
+        self.assertEqual(context.root.children[0].children[1].xsd_element.name, 'p1')
 
     def test_type_substitution(self):
         schema = xmlschema.XMLSchema(dedent("""
@@ -654,6 +663,140 @@ class XMLSchemaProxyTest(xpath_test_class.XPathTestCase):
         self.assertEqual(context.root.children[1].children[3].type_name,
                          '{http://www.w3.org/2001/XMLSchema}string')
 
+    def test_default_values__issue_094(self):
+        schema = xmlschema.XMLSchema(dedent("""
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"
+                    xmlns="http://xpath.test/ns"
+                    targetNamespace="http://xpath.test/ns">
+                <xs:element id="container" name="container"/>
+                <xs:element id="q1" name="q1" type="xs:integer" default="1000"/>
+                <xs:element name="p1" type="pType"/>
+                <xs:complexType name="pType">
+                   <xs:attribute name="p0" type="xs:integer"/>
+                   <xs:attribute name="p1" type="xs:integer" default="1000"/>
+                </xs:complexType>
+            </xs:schema>"""))
+
+        schema_proxy = schema.xpath_proxy
+
+        self.assertTrue(schema.is_valid(
+            '<container xmlns="http://xpath.test/ns"><q1>1000</q1></container>'
+        ))
+        self.assertTrue(schema.is_valid(
+            '<container xmlns="http://xpath.test/ns"><q1>1001</q1></container>'
+        ))
+
+        # Substitution of simple content
+        xml_data = dedent("""\
+            <container xmlns="http://xpath.test/ns">
+                <q1>1000</q1>
+                <q1></q1>
+                <p1/>
+            </container>""")
+
+        self.assertTrue(schema.is_valid(xml_data))
+        root = self.etree.XML(xml_data)
+        namespaces = {'': "http://xpath.test/ns",
+                      'xsi': "http://www.w3.org/2001/XMLSchema-instance"}
+
+        context = XPathContext(root, namespaces=namespaces)
+        context.root.apply_schema(schema_proxy)
+
+        self.assertEqual(context.root.type_name,
+                         '{http://www.w3.org/2001/XMLSchema}anyType')
+        self.assertIsNotNone(context.root.xsd_element)
+        self.assertEqual(context.root.xsd_element.name,
+                         '{http://xpath.test/ns}container')
+
+        self.assertEqual(context.root.children[1].type_name,
+                         '{http://www.w3.org/2001/XMLSchema}integer')
+        self.assertIsNotNone(context.root.children[1].xsd_element)
+        self.assertEqual(context.root.children[1].xsd_element.name,
+                         '{http://xpath.test/ns}q1')
+
+        self.assertEqual(context.root.children[3].type_name,
+                         '{http://www.w3.org/2001/XMLSchema}integer')
+        self.assertIsNotNone(context.root.children[3].xsd_element)
+        self.assertEqual(context.root.children[3].xsd_element.name,
+                         '{http://xpath.test/ns}q1')
+
+        self.assertEqual(context.root.children[1].typed_value, 1000)
+        self.assertEqual(context.root.children[3].typed_value, 1000)
+
+        self.assertEqual(context.root.children[5].type_name,
+                         '{http://xpath.test/ns}pType')
+        self.assertEqual(len(context.root.children[5].attributes), 1)
+        self.assertEqual(context.root.children[5].attributes[0].name, 'p1')
+        self.assertEqual(context.root.children[5].xsd_element.name,
+                         '{http://xpath.test/ns}p1')
+
+    def test_fixed_values__issue_094(self):
+        schema = xmlschema.XMLSchema(dedent("""
+            <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"
+                    xmlns="http://xpath.test/ns"
+                    targetNamespace="http://xpath.test/ns">
+                <xs:element id="container" name="container"/>
+                <xs:element id="q1" name="q1" type="xs:integer" fixed="1000"/>
+                <xs:element name="p1" type="pType"/>
+                <xs:complexType name="pType">
+                   <xs:attribute name="p0" type="xs:integer"/>
+                   <xs:attribute name="p1" type="xs:integer" fixed="1000"/>
+                </xs:complexType>
+            </xs:schema>"""))
+
+        schema_proxy = schema.xpath_proxy
+
+        self.assertTrue(schema.is_valid(
+            '<container xmlns="http://xpath.test/ns"><q1>1000</q1></container>'
+        ))
+        self.assertFalse(schema.is_valid(
+            '<container xmlns="http://xpath.test/ns"><q1>1001</q1></container>'
+        ))
+
+        # Substitution of simple content
+        xml_data = dedent("""\
+            <container xmlns="http://xpath.test/ns">
+                <q1>1000</q1>
+                <q1></q1>
+                <p1/>
+            </container>""")
+
+        self.assertTrue(schema.is_valid(xml_data))
+        root = self.etree.XML(xml_data)
+        namespaces = {'': "http://xpath.test/ns",
+                      'xsi': "http://www.w3.org/2001/XMLSchema-instance"}
+
+        context = XPathContext(root, namespaces=namespaces)
+        context.root.apply_schema(schema_proxy)
+
+        self.assertEqual(context.root.type_name,
+                         '{http://www.w3.org/2001/XMLSchema}anyType')
+        self.assertIsNotNone(context.root.xsd_element)
+        self.assertEqual(context.root.xsd_element.name,
+                         '{http://xpath.test/ns}container')
+
+        self.assertEqual(context.root.children[1].type_name,
+                         '{http://www.w3.org/2001/XMLSchema}integer')
+        self.assertIsNotNone(context.root.children[1].xsd_element)
+        self.assertEqual(context.root.children[1].xsd_element.name,
+                         '{http://xpath.test/ns}q1')
+
+        self.assertEqual(context.root.children[3].type_name,
+                         '{http://www.w3.org/2001/XMLSchema}integer')
+        self.assertIsNotNone(context.root.children[3].xsd_element)
+        self.assertEqual(context.root.children[3].xsd_element.name,
+                         '{http://xpath.test/ns}q1')
+
+        self.assertEqual(context.root.children[1].typed_value, 1000)
+        self.assertEqual(context.root.children[3].typed_value, 1000)
+
+        self.assertEqual(context.root.children[5].type_name,
+                         '{http://xpath.test/ns}pType')
+        self.assertEqual(len(context.root.children[5].attributes), 1)
+        self.assertEqual(context.root.children[5].attributes[0].name, 'p1')
+        self.assertEqual(context.root.children[5].xsd_element.name,
+                         '{http://xpath.test/ns}p1')
+
 
 @unittest.skipIf(xmlschema is None or lxml_etree is None, "both xmlschema and lxml required")
 class LxmlXMLSchemaProxyTest(XMLSchemaProxyTest):
