@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright (c), 2018-2021, SISSA (International School for Advanced Studies).
+# Copyright (c), 2018-2025, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -18,11 +18,12 @@ try:
 except ImportError:
     lxml_etree = None
 
-from elementpath import XPathContext, AttributeNode
-from elementpath.datatypes import Timezone, DateTime10, DateTime, DateTimeStamp, \
-    GregorianDay, GregorianMonth, GregorianMonthDay, GregorianYear10, GregorianYearMonth10, \
-    Duration, YearMonthDuration, DayTimeDuration, Date10, Time, QName, UntypedAtomic, \
-    Base64Binary, HexBinary
+from elementpath import XPathContext, select
+from elementpath.datatypes import Timezone, DateTime, DateTimeStamp, \
+    GregorianDay, GregorianMonth, GregorianMonthDay, GregorianYear10, \
+    GregorianYearMonth10, Duration, YearMonthDuration, DayTimeDuration, \
+    Date10, Time, QName, UntypedAtomic, Base64Binary, HexBinary
+from elementpath.xpath_nodes import TextAttributeNode
 from elementpath.namespaces import XSD_NAMESPACE
 from elementpath.xpath_tokens import XPathConstructor
 
@@ -322,19 +323,19 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         tz1 = Timezone(datetime.timedelta(hours=5, minutes=24))
         self.check_value('xs:dateTime(())', [])
         self.check_value('xs:dateTime("1969-07-20T20:18:00")',
-                         DateTime10(1969, 7, 20, 20, 18))
+                         DateTime(1969, 7, 20, 20, 18))
         self.check_value('xs:dateTime(xs:untypedAtomic("1969-07-20T20:18:00"))',
-                         DateTime10(1969, 7, 20, 20, 18))
+                         DateTime(1969, 7, 20, 20, 18))
         self.check_value('xs:dateTime("2000-05-10T21:30:00+05:24")',
                          datetime.datetime(2000, 5, 10, hour=21, minute=30, tzinfo=tz1))
         self.check_value('xs:dateTime("1999-12-31T24:00:00")',
                          datetime.datetime(2000, 1, 1, 0, 0))
         self.check_value('xs:dateTime(xs:date("1969-07-20"))',
-                         DateTime10(1969, 7, 20))
+                         DateTime(1969, 7, 20))
 
-        self.check_value('xs:dateTime(xs:date("1969-07-20"))', DateTime10)
+        self.check_value('xs:dateTime(xs:date("1969-07-20"))', DateTime)
         with self.assertRaises(AssertionError):
-            self.check_value('xs:dateTime(xs:date("1969-07-20"))', DateTime)
+            self.check_value('xs:dateTime(xs:date("1969-07-20"))')
 
         self.parser._xsd_version = '1.1'
         try:
@@ -354,16 +355,16 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
 
         root = self.etree.XML('<root a="1969-07-20T20:18:00"/>')
         context = XPathContext(root)
-        self.check_value('xs:dateTime(@a)', DateTime10(1969, 7, 20, 20, 18), context=context)
+        self.check_value('xs:dateTime(@a)', DateTime(1969, 7, 20, 20, 18), context=context)
 
-        context.item = AttributeNode('a', str(DateTime10(1969, 7, 20, 20, 18)))
-        self.check_value('xs:dateTime(.)', DateTime10(1969, 7, 20, 20, 18), context=context)
+        context.item = TextAttributeNode('a', str(DateTime(1969, 7, 20, 20, 18)))
+        self.check_value('xs:dateTime(.)', DateTime(1969, 7, 20, 20, 18), context=context)
 
-        context.item = AttributeNode('a', 'true')
+        context.item = TextAttributeNode('a', 'true')
         self.check_value('xs:dateTime(.)', ValueError, context=context)
 
-        context.item = DateTime10(1969, 7, 20, 20, 18)
-        self.check_value('xs:dateTime(.)', DateTime10(1969, 7, 20, 20, 18), context=context)
+        context.item = DateTime(1969, 7, 20, 20, 18)
+        self.check_value('xs:dateTime(.)', DateTime(1969, 7, 20, 20, 18), context=context)
 
     def test_datetimestamp_constructor(self):
         tz0 = Timezone(datetime.timedelta(hours=7, minutes=0))
@@ -434,6 +435,12 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         self.check_value('xs:date(@a)', Date10(2017, 1, 19), context=context)
 
         class DummyXsdDateType(xpath_test_class.DummyXsdType):
+            name = local_name = None
+            xsd_version = '1.0'
+
+            def is_list(self): pass
+            def is_union(self): pass
+
             def is_simple(self):
                 return True
 
@@ -441,16 +448,16 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
                 return Date10.fromstring(obj)
 
             def validate(self, obj, *args, **kwargs):
-                if not isinstance(obj, Date10):
-                    raise TypeError()
+                Date10.validate(obj)
 
-        context.item = AttributeNode('a', 'true', xsd_type=DummyXsdDateType())
-        self.check_value('xs:date(.)', TypeError, context=context)
+        context.item = TextAttributeNode('a', 'true')
+        context.item.xsd_type = DummyXsdDateType()
+        self.check_value('xs:date(.)', ValueError, context=context)
 
-        context.item = AttributeNode('a', str(Date10(2017, 1, 19)))
+        context.item = TextAttributeNode('a', str(Date10(2017, 1, 19)))
         self.check_value('xs:date(.)', Date10(2017, 1, 19), context=context)
 
-        context.item = AttributeNode('a', 'true')
+        context.item = TextAttributeNode('a', 'true')
         self.check_value('xs:date(.)', ValueError, context=context)
 
         root = self.etree.XML("<root>2017-10-02</root>")
@@ -702,6 +709,25 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
                         'no constructor function exists for xs:NOTATION')
         self.wrong_name('"A120" castable as xs:NOTATION', 'XPST0080')
 
+    def test_missing_context_on_namespaced_name__issue_068(self):
+        namespaces = {
+            'test': "urn:example:names:common-test-names"
+        }
+
+        root = self.etree.XML("""
+        <A xmlns:test="urn:example:names:common-test-names">
+            <B>
+                <test:number>1</test:number>
+            </B>
+        </A>
+        """)
+
+        context_node = select(root, "B", namespaces=namespaces)[0]
+        result = select(root, "xs:decimal(./test:number)", namespaces, item=context_node)
+        self.assertEqual(result, 1)
+        result = select(root, "xs:decimal(test:number)", namespaces, item=context_node)
+        self.assertEqual(result, 1)
+
 
 @unittest.skipIf(lxml_etree is None, "The lxml library is not installed")
 class LxmlXPath2ConstructorsTest(XPath2ConstructorsTest):
